package com.ruoyi.jjh.ent.service.impl;

import cn.hutool.core.util.RandomUtil;
import cn.hutool.crypto.digest.DigestAlgorithm;
import cn.hutool.crypto.digest.Digester;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.ruoyi.jjh.ent.service.SmsAlertsService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.time.Instant;
import java.util.HashMap;
import java.util.Map;

/**
 * 短信提醒业务处理层
 *
 * @author du
 * @since 2024/7/16 13:26
 */
@Service
public class SmsAlertsServiceImpl implements SmsAlertsService {

    @Value("${smsSend}")
    private String smsSend;


    @Value("${smsPhoneAdd}")
    private String smsPhoneAdd;

    @Value("${paaSID}")
    private String paaSID;

    @Value("${smsToken}")
    private String smsToken;

    @Value("${code}")
    private String code;

    @Value("${smsV}")
    private String smsV;

    @Value("${paramType}")
    private String paramType;

    /**
     * 生成短信统一的请求头
     *
     * @return 请求头
     */
    @Override
    public HashMap<String, String> getMap() {
        Digester sha256 = new Digester(DigestAlgorithm.SHA256);
        String timestamp = String.valueOf(Instant.now().getEpochSecond());
        String str = RandomUtil.randomString(30);
        //请求头
        HashMap<String, String> headers = new HashMap<>();//存放请求头，可以存放多个请求头
        headers.put("x-tif-paasid", paaSID);
        headers.put("x-tif-signature", sha256.digestHex(timestamp + smsToken + str + timestamp).toUpperCase());
        headers.put("x-tif-timestamp", timestamp);
        headers.put("x-tif-nonce", str);
        return headers;
    }

    /**
     * 发送短信
     *
     * @param param1
     * @param param2
     * @return
     */
    @Override
    public String enterSms(String param1, String param2, String numbers) {
        JSONObject jsonObject = new JSONObject();//存放参数
        //请求体
        JSONObject params = new JSONObject();//存放参数
        params.set("param1", param1);
        params.set("param2", param2);
        jsonObject.set("type", paramType);
        jsonObject.set("empowerCode", smsV);
        jsonObject.set("numbers", numbers);
        jsonObject.set("params", params);
        HttpResponse response = HttpRequest.post(smsSend)
                .addHeaders(getMap())
                .body(jsonObject.toString())
                .execute();
        JSONObject jsonObj = JSONUtil.parseObj(response.body());
        return String.valueOf(jsonObj.get("msg"));
    }


    /**
     * 短信接收人号码新增
     *
     * @return 所有数据
     */
    @Override
    public String addSsmPhone(String nums) {
        Map<String, Object> js1 = new HashMap<>();//存放参数
        js1.put("orderId", code);
        js1.put("nums", nums);
        HttpResponse response = HttpRequest.post(smsPhoneAdd)
                .addHeaders(getMap())
                .form(js1)
                .execute();
        JSONObject jsonObj = JSONUtil.parseObj(response.body());
        return String.valueOf(jsonObj.get("msg"));
    }
}
